<?php
/**
 * @package		Advanced Tags & Prefixes
 * @author		Ryan Hoerr
 * @copyright	(c) 2012 Ryan Hoerr / Sublime Development
 * @license		http://www.sublimism.com/products/terms-of-use
 * @version		$Id: prefixes.php 35 2012-12-11 18:06:43Z No1 1000 $
 * @updated		$Date: 2012-12-11 13:06:43 -0500 (Tue, 11 Dec 2012) $
 */

/**
 * Do all the ACP Prefix management actions and views
 */

if ( ! defined( 'IN_ACP' ) ) {
	print "<h1>Incorrect access</h1>You cannot access this file directly. If you have recently upgraded, make sure you upgraded 'admin.php'.";
	exit();
}

class admin_advancedtagsprefixes_prefixes_prefixes extends ipsCommand
{
	private $html;
	private $form_code;
	private $form_code_js;
	private $han_editor;

	public function doExecute( ipsRegistry $registry ) 
	{
		//-----------------------------------------
		// Load skin
		//-----------------------------------------
		
		$this->html			= $this->registry->output->loadTemplate('cp_skin_prefixes');
		
		//-----------------------------------------
		// Set up stuff
		//-----------------------------------------
		
		$this->form_code	= $this->html->form_code	= 'module=prefixes&amp;section=prefixes';
		$this->form_code_js	= $this->html->form_code_js	= 'module=prefixes&section=prefixes';

		$classToLoad = IPSLib::loadLibrary( IPSLib::getAppDir( 'advancedtagsprefixes' ) . "/sources/classes/common.php", 'sldTopicPrefixes_common', 'advancedtagsprefixes' );
		$this->common = new $classToLoad( $registry );
		
		//-----------------------------------------
		// Load lang
		//-----------------------------------------
		
		$this->registry->class_localization->loadLanguageFile( array( 'admin_global' ), 'advancedtagsprefixes' );

		///-----------------------------------------
		// What to do...
		//------------------------------------------
		
		$this->prefixes = $this->cache->getCache('topic_prefixes');
		
		switch( $this->request['do'] )
		{
			default:
			case 'display':
				$this->_showAll();
			break;
			
			case 'addnew':
				$this->registry->getClass('class_permissions')->checkPermissionAutoMsg( 'prefix_addnew' );
				$this->_add();
			break;
			
			case 'edit':
				$this->registry->getClass('class_permissions')->checkPermissionAutoMsg( 'prefix_edit' );
				$this->_edit();
			break;
			
			case 'delete':
				$this->registry->getClass('class_permissions')->checkPermissionAutoMsg( 'prefix_delete' );
				$this->_delete();
			break;
			
			case 'doAdd':
				$this->registry->getClass('class_permissions')->checkPermissionAutoMsg( 'prefix_addnew' );
				$this->_doAdd();
			break;
			
			case 'doEdit':
				$this->registry->getClass('class_permissions')->checkPermissionAutoMsg( 'prefix_edit' );
				$this->_doEdit();
			break;
		}
		
		//-----------------------------------------
		// Pass to CP output hander
		//-----------------------------------------
		
		$this->registry->output->html_main .= $this->registry->output->global_template->global_frame_wrapper();
		$this->registry->output->sendOutput();
		
	}
	
	/**
	 * Show the index [all prefixes we've created]
	 */
	protected function _showAll()
	{
		/**
		 * I'm told this query won't play nice in anything but MySQL,
		 * so we'll try to make the drivers happy.
		 */
		if($this->settings['sql_driver'] == 'mysql') {
			$this->DB->build( array(	'select'	=> 'p.*',
										'from'		=> array('topic_prefixes' => 'p'),
										'group'		=> 'prefix_id',
										'order'		=> 'prefix_title asc',
										'add_join'	=> array( array(
												'select'	=> 'count(f.id) as forums',
												'from'		=> array('forums' => 'f'),
												'type'		=> 'left',
												'where'		=> "f.tag_predefined like concat('%',p.prefix_title,'%')",
											) )
										) );
		}
		else {
			$this->DB->build( array(	'select'	=> "p.*, '-' as forums",
										'from'		=> array('topic_prefixes' => 'p'),
										'group'		=> 'prefix_id',
										'order'		=> 'prefix_title asc' ) );
		}
		$this->DB->execute();
		
		/**
		 * Do the fetching and displaying
		 */
		$dropdown = array( array( 0, $this->lang->words['pre_select_one'] ) );
		while( $r = $this->DB->fetch() ) {
			$prefixes[ $r['prefix_id'] ]	= $r;
			$dropdown[] = array( $r['prefix_id'], $r['prefix_title'] );
		}
		
		/**
		 * Refresh the prefixes cache
		 */
		$this->cache->setCache( 'topic_prefixes', $prefixes,  array( 'array' => 1 ) );

		/**
		 * Make sure everything is cool.
		 */
		$warnings = $this->common->checkState();
		
		
		$this->registry->output->html .= $this->html->prefixMainScreen( $prefixes, $dropdown, $warnings );
	}
	
	/**
	 * Add-a-prefix form
	 */
	protected function _add()
	{
		$this->registry->output->html .= $this->html->prefixEditForm( );
	}
	
	/**
	 * Change-a-prefix form
	 */
	protected function _edit()
	{
		$prefix_id = intval( $this->request['p'] );
		
		if( $prefix_id < 1 )
			$this->registry->output->showError( $this->lang->words['pre_invalid_id'], 17002 );
		
		/**
		 * Get the current data
		 */
		$data = $this->DB->buildAndFetch( array(	'select'	=> '*',
													'from'		=> 'topic_prefixes',
													'where'		=> 'prefix_id=' . $prefix_id ) );
		
		/**
		 * Find the forums it's enabled in
		 */
		$this->DB->build( array(	'select'	=> 'id, tag_predefined',
									'from'		=> 'forums',
									'where'		=> "tag_predefined like '%" . IPSText::safeslashes( $data['prefix_title'] ) . "%'" ) );
		$this->DB->execute();
		while( $r = $this->DB->fetch() ) {
			$tags = array_filter( explode( ',', $r['tag_predefined'] ) );
			if( count( $tags ) ) {
				foreach( $tags as $tag ) {
					if( trim( $tag ) == $data['prefix_title'] ) {
						$data['forums'][] = $r['id'];
					}
				}
			}
		}
		
		/**
		 * Goooo
		 */
		$this->registry->output->html .= $this->html->prefixEditForm( $data );
	}
	
	/**
	 * Whack-a-prefix
	 */
	protected function _delete()
	{
		$prefix_id = intval($this->request['p']);
		
		if( $prefix_id < 1)
			$this->registry->output->showError( $this->lang->words['pre_invalid_id'], 17002 );
		
		/**
		 * Remove all fingerprints; we don't want to be found...
		 */
		$this->common->_removeFromForums( $this->prefixes[ $prefix_id ]['prefix_title'] );
		
		$this->DB->delete( 'topic_prefixes', "prefix_id=$prefix_id" );
		
		/**
		 * Goooo
		 */
		$this->registry->adminFunctions->saveAdminLog( sprintf( $this->lang->words['pre_log_deleted'], $this->prefixes[ $prefix_id ]['prefix_title'] ) );
		$this->registry->output->redirect( $this->settings['base_url'] . $this->form_code, $this->lang->words['pre_did_delete'] );
	}
	
	/**
	 * Add a prefix [submit]
	 */
	protected function _doAdd()
	{
		/**
		 * Take care of data validation
		 * Inputs: title, pre, post, showtitle, groups[], forums[]
		 */
		$db_insert = array();
		$db_insert['prefix_title']		= trim( $this->common->_cleanPrefix( IPSText::safeslashes( $this->request['title'] ) ) );
		$db_insert['prefix_pre']		= trim( IPSText::safeslashes( $_POST['pre'] ) );
		$db_insert['prefix_post']		= trim( IPSText::safeslashes( $_POST['post'] ) );
		$db_insert['prefix_showtitle']	= !((bool) $this->request['showtitle']);
		
		if( count( $this->request['groups'] ) ) {
			foreach( $this->request['groups'] as $group ) {
				if( intval($group) > 0 )
					$db_insert['prefix_groups'][] = intval($group);
			}
			$db_insert['prefix_groups']	= implode(',', $db_insert['prefix_groups']);
		}
		
		if( $db_insert['prefix_title'] == '' )
			$this->registry->output->showError( $this->lang->words['pre_need_title'], 17001 );
		
		$r	= $this->DB->buildAndFetch( array(	'select'	=> 'prefix_id',
												'from'		=> 'topic_prefixes',
												'where'		=> "prefix_title='" . $db_insert['prefix_title'] . "'" ) );
		if( !empty($r) && intval($r['prefix_id']) > 0 )
			$this->registry->output->showError( $this->lang->words['pre_already_exists'], 17010 );
		
		if( IPSText::mbstrlen($db_insert['prefix_title']) > 255 || IPSText::mbstrlen($db_insert['prefix_pre']) > 255 || IPSText::mbstrlen($db_insert['prefix_post']) > 255)
			$this->registry->output->showError( $this->lang->words['pre_long_field'], 17003 );
		
		/**
		 * Insert the new prefix
		 */
		$this->DB->insert( 'topic_prefixes', $db_insert );
		$prefix_id = $this->DB->getInsertId();
		
		/**
		 * Add the prefix to the selected forums
		 */
		$this->common->_addToForums( $this->request['title'], $this->request['forums'] );
		
		/**
		 * Goooo
		 */
		$this->registry->adminFunctions->saveAdminLog( sprintf( $this->lang->words['pre_log_added'], $db_insert['prefix_title'] ) );
		$this->registry->output->redirect( $this->settings['base_url'] . $this->form_code, $this->lang->words['pre_did_add'] );
	}
	
	/**
	 * Edit a prefix [submit]
	 */
	protected function _doEdit()
	{
		/**
		 * Take care of data validation
		 * Inputs: title, pre, post, showtitle, groups[], forums[]
		 */
		$db_insert = array();
		$db_insert['prefix_title']		= trim( $this->common->_cleanPrefix( IPSText::safeslashes( $this->request['title'] ) ) );
		$db_insert['prefix_pre']		= trim( IPSText::safeslashes( $_POST['pre'] ) );
		$db_insert['prefix_post']		= trim( IPSText::safeslashes( $_POST['post'] ) );
		$db_insert['prefix_showtitle']	= !((bool) $this->request['showtitle']);
		
		$prefix_id = intval($this->request['prefix_id']);
		
		if( count( $this->request['groups'] ) ) {
			foreach( $this->request['groups'] as $group ) {
				if( intval($group) > 0 )
					$db_insert['prefix_groups'][] = intval($group);
			}
			$db_insert['prefix_groups']	= implode(',', $db_insert['prefix_groups']);
		}
		
		if( $prefix_id < 1)
			$this->registry->output->showError( $this->lang->words['pre_invalid_id'], 17002 );
		
		if( $db_insert['prefix_title'] == '')
			$this->registry->output->showError( $this->lang->words['pre_need_title'], 17001 );

		if( IPSText::mbstrlen($db_insert['prefix_title']) > 255 || IPSText::mbstrlen($db_insert['prefix_pre']) > 255 || IPSText::mbstrlen($db_insert['prefix_post']) > 255)
			$this->registry->output->showError( $this->lang->words['pre_long_field'], 17003 );
		
		$r	= $this->DB->buildAndFetch( array(	'select'	=> 'prefix_id',
												'from'		=> 'topic_prefixes',
												'where'		=> "prefix_title='" . $db_insert['prefix_title'] . "'" ) );
		if( !empty($r) && intval($r['prefix_id']) != $prefix_id )
			$this->registry->output->showError( $this->lang->words['pre_already_exists'], 17010 );
		
		/**
		 * Update the prefix
		 */
		$this->DB->update( 'topic_prefixes', $db_insert, 'prefix_id=' . $prefix_id );
		
		/**
		 * Update existing instances [if necessary]
		 */
		if( $db_insert['prefix_title'] != $this->prefixes[ $prefix_id ]['prefix_title'] ) {
			$this->common->updateTags( $this->prefixes[ $prefix_id ]['prefix_title'], $db_insert['prefix_title'] );
		}
		
		/**
		 * Clean up the forum records. This is a little ungraceful, but it works...
		 * 
		 * We need to remove it from every forum before we update them--
		 * otherwise, you'd never be able to remove it from a forum at all.
		 */
		$this->common->_removeFromForums( $this->prefixes[ $prefix_id ]['prefix_title'] );
		$this->common->_addToForums( $this->request['title'], $this->request['forums'] );
		
		/**
		 * Goooo
		 */
		$this->registry->adminFunctions->saveAdminLog( sprintf( $this->lang->words['pre_log_edited'], $this->prefixes[ $prefix_id ]['prefix_title'] ) );
		$this->registry->output->redirect( $this->settings['base_url'] . $this->form_code, $this->lang->words['pre_did_edit'] );
	}
}
