<?php
/**
 * @package		Advanced Tags & Prefixes
 * @author		Ryan Hoerr
 * @copyright	(c) 2012 Ryan Hoerr / Sublime Development
 * @license		http://www.sublimism.com/products/terms-of-use
 * @version		$Id: tools.php 37 2013-05-16 03:08:24Z No1_1000 $
 * @updated		$Date: 2013-05-15 23:08:24 -0400 (Wed, 15 May 2013) $
 */

/**
 * Do all the other ACP Prefix management actions and views
 */

if ( ! defined( 'IN_ACP' ) ) {
	print "<h1>Incorrect access</h1>You cannot access this file directly. If you have recently upgraded, make sure you upgraded 'admin.php'.";
	exit();
}

class admin_advancedtagsprefixes_tools_tools extends ipsCommand
{
	private $html;
	private $form_code;
	private $form_code_js;
	private $han_editor;

	public function doExecute( ipsRegistry $registry ) 
	{
		//-----------------------------------------
		// Load skin
		//-----------------------------------------
		
		$this->html			= $this->registry->output->loadTemplate('cp_skin_tools');
		
		//-----------------------------------------
		// Set up stuff
		//-----------------------------------------

		$this->form_code	= $this->html->form_code	= 'module=tools&amp;section=tools';
		$this->form_code_js	= $this->html->form_code_js	= 'module=tools&section=tools';
		
		$classToLoad = IPSLib::loadLibrary( IPSLib::getAppDir( 'advancedtagsprefixes' ) . "/sources/classes/common.php", 'sldTopicPrefixes_common', 'advancedtagsprefixes' );
		$this->common = new $classToLoad( $registry );

		//-----------------------------------------
		// Load lang
		//-----------------------------------------
		
		$this->registry->class_localization->loadLanguageFile( array( 'admin_global' ), 'advancedtagsprefixes' );

		///-----------------------------------------
		// What to do...
		//------------------------------------------
		
		$this->prefixes = $this->cache->getCache('topic_prefixes');
		
		switch( $this->request['do'] )
		{
			default:
			case 'display':
				$this->_showAll();
			break;

			case 'fixAllTheThings':
				$this->_fixAll();
			break;

			case 'addTags':
				$this->_addTags();
			break;

			case 'updateTags':
				$this->_updateTags();
			break;

			case 'deleteTags':
				$this->_deleteTags();
			break;

			case 'lowerTags':
				$this->_lowerTags();
			break;

			case 'rebuildCache':
				$this->_rebuildCache();
			break;
		}
		
		//-----------------------------------------
		// Pass to CP output hander
		//-----------------------------------------
		
		$this->registry->output->html_main .= $this->registry->output->global_template->global_frame_wrapper();
		$this->registry->output->sendOutput();
		
	}
	
	/**
	 * Show the index
	 */
	protected function _showAll( $message = '' )
	{
		$dropdown = array();
		$prefixes = array();

		/**
		 * Assume nothing.
		 */
		if( $this->DB->checkForTable('topic_prefixes') ) {
			$this->DB->build( array(	'select'	=> '*',
										'from'		=> 'topic_prefixes',
										'order'		=> 'prefix_title asc' ) );
			$this->DB->execute();
			
			/**
			 * Do the fetching and displaying
			 */
			$dropdown = array( array( 0, $this->lang->words['pre_select_one'] ) );
			while( $r = $this->DB->fetch() ) {
				$prefixes[ $r['prefix_id'] ]	= $r;
				$dropdown[] = array( $r['prefix_id'], $r['prefix_title'] );
			}
		
			/**
			 * Refresh the prefixes cache
			 */
			$this->cache->setCache( 'topic_prefixes', $prefixes,  array( 'array' => 1 ) );
		}

		/**
		 * Make sure everything is cool.
		 */
		$warnings = $this->common->checkState();
		
		$this->registry->output->html .= $this->html->prefixMainScreen( $prefixes, $dropdown, $warnings, $message );
	}

	/**
	 * Attempt to identify and fix a number of common database errors
	 * including missing columns [from a botched upgrade] and collation
	 * conflicts.
	 */
	protected function _fixAll()
	{
		$pre = trim( ipsRegistry::dbFunctions()->getPrefix() );
		$did[] = "<b>Looking for things to fix...</b>";

		/**
		 * Check for table and columns
		 */
		
		if( !$this->DB->checkForTable('topic_prefixes') ) {
			// How the heck does this happen?
			$this->DB->query( "CREATE TABLE IF NOT EXISTS {$pre}topic_prefixes (
				  prefix_id int(10) NOT NULL AUTO_INCREMENT,
				  prefix_title varchar(255) DEFAULT NULL,
				  prefix_pre varchar(255) DEFAULT NULL,
				  prefix_post varchar(255) DEFAULT NULL,
				  prefix_groups mediumtext,
				  prefix_showtitle tinyint(1) NOT NULL DEFAULT '1',
				  PRIMARY KEY (prefix_id),
				  KEY title (prefix_title)
				)" );
			$did[] = "Created missing topic_prefixes table.";
		}
		else {
			$did[] = "Table topic_prefixes appears okay.";
		}

		$columns[] = array( 'forums', 'require_prefix', 'tinyint(1)', '0' );
		$columns[] = array( 'forums', 'default_prefix', 'varchar(255)', "''" );
		$columns[] = array( 'forums', 'default_tags', 'mediumtext' );
		$columns[] = array( 'forums', 'show_prefix_in_desc', 'tinyint(1)', '0' );
		$columns[] = array( 'forums', 'tag_mode', 'varchar(255)', "'inherit'" );
		$columns[] = array( 'forums', 'newest_prefix', 'varchar(2550)' );
		$columns[] = array( 'topic_mmod', 'topic_prefix', 'int(10)', '-1' );
		$columns[] = array( 'topic_mmod', 'topic_add_tags', 'mediumtext' );

		$fixed = 0;
		foreach( $columns as $column ) {
			if( !$this->DB->checkForField( $column[1], $column[0] ) ) {
				$this->DB->addField( $column[0], $column[1], $column[2], (!empty($column[3]) ? $column[3] : null) );
				$fixed++;
			}
		}
		$did[] = "Added $fixed missing columns; ".(count($columns) - $fixed)." were okay.";
		
		/**
		 * Check collations
		 */
		$this->DB->query( "show full columns from {$pre}topic_prefixes where Field='prefix_title'" );
		$r = $this->DB->fetch();
		$coll_ours = $r['Collation'];

		$this->DB->query( "show full columns from {$pre}forums where Field='tag_predefined'" );
		$r = $this->DB->fetch();
		$coll_theirs = $r['Collation'];
		$did[] = "Tags & Prefixes collation appears to be $coll_ours... forums collation is $coll_theirs.";

		if( $coll_ours != $coll_theirs ) {
			$did[] = "Mismatch! Converting prefixes collation to $coll_theirs.";
			$charset = substr( $coll_theirs, 0, strpos( $coll_theirs, '_' ) );

			$this->DB->query( "ALTER TABLE {$pre}topic_prefixes
				MODIFY prefix_title VARCHAR(255) CHARACTER SET {$charset} COLLATE {$coll_theirs},
				MODIFY prefix_pre VARCHAR(255) CHARACTER SET {$charset} COLLATE {$coll_theirs},
				MODIFY prefix_post VARCHAR(255) CHARACTER SET {$charset} COLLATE {$coll_theirs},
				MODIFY prefix_groups mediumtext CHARACTER SET {$charset} COLLATE {$coll_theirs}" );
		}
		else {
			$did[] = "Everything seems okay.";
		}

		$did[] = "If you still have problems, go ask for support.";

		/**
		 * Show page and messages
		 */
		$this->registry->adminFunctions->saveAdminLog( $this->lang->words['pre_fixall_log'] );
		$this->_showAll( implode( $did, "<br />" ) );
	}

	/**
	 * Mass-add tags based on title and forum search.
	 * First, show them the results and confirm.
	 */
	protected function _addTags()
	{
		$find			= IPSText::safeslashes( $this->request['find'] );
		$remove			= IPSText::safeslashes( $this->request['removeText'] );
		$remove_seo		= IPSText::makeSeoTitle( $remove );
		$tags_add		= explode( ',', IPSText::safeslashes( $this->request['replace_text'] ) );
		$prefix_add		= intval( $this->request['replace'] );
		$per_cycle		= intval( $this->request['perCycle'] );
		$start 			= intval( $this->request['start'] );
		$forums			= array();
		$where			= array();
		$topics			= array();

		if( !is_array($this->request['forums']) ) {
			$this->request['forums'] = explode( ',', $this->request['forums'] );
		}

		if( count( $tags_add ) ) {
			foreach( $tags_add as $k => $v ) {
				$tags_add[$k] = $this->settings['tags_force_lower'] ? IPSText::mbstrtolower( trim($v) ) : trim($v);
			}
			$this->request['replace_text'] = implode( ',', $tags_add );
		}

		if( count( $this->request['forums'] ) ) {
			foreach( $this->request['forums'] as $forum ) {
				if( intval($forum) > 0 )
					$forums[] = intval($forum);
			}
		}

		if( ( empty($find) && !count($forums) ) || ( empty($tags_add) && $prefix_add < 1 ) || $per_cycle < 1 ) {
			$this->registry->output->redirect( $this->settings['base_url'] . $this->form_code, $this->lang->words['pre_missing_data'] );
		}

		if( !empty($find) ) {
			if( $this->request['incOP'] ) {
				$where[] = "(title like '%" . $find . "%' OR p.post like '%" . $find . "%')";
			}
			else {
				$where[] = "title like '%" . $find . "%'";
			}
		}
		if( count($forums) ) {
			$where[] = 'forum_id in (' . implode( ',', $forums ) . ')';
		}
		if( $this->request['incPinned'] != 1 ) {
			$where[] = 'pinned=0';
		}
		if( $this->request['incLocked'] != 1 ) {
			$where[] = "state!='closed'";
		}


		/**
		 * Search
		 */
		$num = $this->request['confirmed'] ? $per_cycle : '99999999999';
		if( $this->request['incOP'] ) {
			array_unshift( $where, 'p.pid=t.topic_firstpost' );
			$this->DB->build( array(	'select'	=> 'tid, title, starter_name, start_date',
										'from'		=> array( 'posts' => 'p', 'topics' => 't' ),
										'where'		=> implode( ' and ', $where ),
										'order'		=> 'tid desc',
										'limit'		=> array( $start, $num ),
										'add_join'	=> array( array(
													'select'	=> 'name',
													'from'		=> array('forums' => 'f'),
													'type'		=> 'left',
													'where'		=> 'f.id=t.forum_id',
												) )
											) );
		}
		else {
			$this->DB->build( array(	'select'	=> 'tid, title, starter_name, start_date',
										'from'		=> array( 'topics' => 't' ),
										'where'		=> implode( ' and ', $where ),
										'order'		=> 'tid desc',
										'limit'		=> array( $start, $num ),
										'add_join'	=> array( array(
													'select'	=> 'name',
													'from'		=> array('forums' => 'f'),
													'type'		=> 'left',
													'where'		=> 'f.id=t.forum_id',
												) )
											) );
		}
		$this->DB->execute();
		while( $r = $this->DB->fetch() ) {
			$topics[] = $r;
		}

		if( $this->request['confirmed'] != 1 ) {
			$this->registry->output->html .= $this->html->showAddConfirm( $topics, $this->prefixes[ $prefix_add ]['prefix_title'] );
		}
		else {
			/* Load tagging stuff */
			if ( ! $this->registry->isClassLoaded('tags') )
			{
				require_once( IPS_ROOT_PATH . 'sources/classes/tags/bootstrap.php' );/*noLibHook*/
				$this->registry->setClass( 'tags', classes_tags_bootstrap::run( 'forums', 'topics' ) );
			}
			
			if( count( $topics ) > 0 ) {
				foreach( $topics as $r ) {
					/**
					 * Get topic tags
					 */
					$tags = $this->registry->getClass('tags')->getTagsByMetaId( array( 'meta_id' => $r['tid'] ) );

					if( !count($tags) ) {
						$tags = array( 'tags' => array() );
					}

					/**
					 * Add in the addition(s) and send back.
					 */
					if( !empty($tags['prefix']) || $prefix_add > 0 ) {
						$_REQUEST['ipsTags_prefix'] = 1;
					}
					else {
						$_REQUEST['ipsTags_prefix'] = 0;
					}

					if( $prefix_add > 0 ) {
						$prefix_text = $this->settings['tags_force_lower'] ? IPSText::mbstrtolower( $this->prefixes[ $prefix_add ]['prefix_title'] ) : $this->prefixes[ $prefix_add ]['prefix_title'];
						array_unshift( $tags['tags'], $prefix_text );
					}
					if( !empty($tags_add) ) {
						$tags['tags'] = array_merge( $tags['tags'], $tags_add );
					}

					/**
					 * Try to make sure no blank tags make it through.
					 */
					$tags['tags'] = array_filter( $tags['tags'] );

					$this->registry->getClass('tags')->replace( $tags['tags'], array(	'meta_id'			=> $r['tid'],
																						'meta_parent_id'	=> $r['forum_id'] ) );
				}
			}

			if( !empty($remove) ) {
				/**
				 * Do a search-and-replace on topic titles to strip $remove.
				 */
				$this->DB->build( array(	'update'	=> 'topics',
											'set'		=> "title=trim( replace( replace(title, '" . strtolower( $remove ) . "', '') , '" . $remove . "', '') ), title_seo=trim( leading '-' from replace(title_seo, '" . $remove_seo . "', '') )",
											'where'		=> "title like '" . $remove . "%'",
											'limit'		=> $num ) );
				$this->DB->execute();
			}
			
			// First iteration - log.
			if( $start == 0 ) {
				$this->registry->adminFunctions->saveAdminLog( sprintf( $this->lang->words['pre_massadd_log'], intval($this->request['total']) ) );
			}

			// Last iteration - all done.
			if( count($topics) < $per_cycle ) {
				$this->registry->output->redirect( $this->settings['base_url'] . $this->form_code, sprintf( $this->lang->words['pre_massadd_resp'], $start + count($topics) ) );
			}

			// If not done, show the update screen.
			$url  = "{$this->settings['base_url']}{$this->form_code}&do=addTags&find=".$find.'&replace='.$this->request['replace'].'&replace_text='.$this->request['replace_text'].
					'&perCycle='.$per_cycle.'&total='.$this->request['total'].'&incOP='.$this->request['incOP'].
					'&incPinned='.$this->request['incPinned'].'&incLocked='.$this->request['incLocked'].
					'&forums='.implode(',', $forums).'&confirmed=1&start='.($start+count($topics));
			$text = sprintf( $this->lang->words['pre_massadd_progress'], $start + count($topics), $this->request['total'], $url );
			$this->registry->output->html .= $this->registry->output->global_template->temporaryRedirect( $url, $text );
		}
	}

	/**
	 * Update tags matching something to be something else.
	 */
	protected function _updateTags()
	{
		$find			= IPSText::safeslashes( $this->request['find'] );
		$replace_text	= IPSText::safeslashes( $this->request['replace_text'] );
		$replace		= intval( $this->request['replace'] );

		if( empty($replace_text) && $replace > 0 ) {
			$replace_text = $this->prefixes[ $replace ]['prefix_title'];
		}

		if( empty($replace_text) || empty($find) ) {
			$this->registry->output->redirect( $this->settings['base_url'] . $this->form_code, $this->lang->words['pre_missing_data'] );
		}

		$count = $this->common->updateTags( $find, $replace_text );

		$this->registry->adminFunctions->saveAdminLog( sprintf( $this->lang->words['pre_massupdate_log'], $count, $find, $replace_text ) );
		$this->registry->output->redirect( $this->settings['base_url'] . $this->form_code, sprintf( $this->lang->words['pre_massupdate_resp'], $count ) );
	}

	/**
	 * Remove tags matching something.
	 */
	protected function _deleteTags()
	{
		$find	= $this->request['find'];

		if( empty($find) ) {
			$this->registry->output->redirect( $this->settings['base_url'] . $this->form_code, $this->lang->words['pre_missing_data'] );
		}

		$count	= $this->common->deleteTags( $find );

		$this->registry->adminFunctions->saveAdminLog( sprintf( $this->lang->words['pre_massdelete_log'], $count, $find ) );
		$this->registry->output->redirect( $this->settings['base_url'] . $this->form_code, sprintf( $this->lang->words['pre_massdelete_resp'], $count, $find ) );
	}

	/**
	 * Convert all tags to lowercase.
	 */
	protected function _lowerTags()
	{
		$pre = trim( ipsRegistry::dbFunctions()->getPrefix() );

		$this->DB->query( "update {$pre}core_tags set tag_text=lower(tag_text)" );
		$this->DB->query( "update {$pre}core_tags_cache set tag_cache_text=lower(tag_cache_text)" );

		// TODO: Langify and log this.
		$this->registry->adminFunctions->saveAdminLog( "Converted all tags/prefixes to lowercase." );
		$this->registry->output->redirect( $this->settings['base_url'] . $this->form_code, "Converted all tags/prefixes to lowercase." );
	}

	/**
	 * Rebuild the tag cache.
	 * Cache generation is based on _addCache() in tags abstract.
	 */
	protected function _rebuildCache()
	{
		$perCycle	= 25;
		$sum		= intval($this->request['sum']);
		$start		= intval($this->request['st']);
		$total		= intval($this->request['total']);

		// Clear existing cache entries
		if( $start == 0 ) {
			$this->DB->delete( 'core_tags_cache' );

			$t = $this->DB->buildAndFetch( array(	'select'	=> 'count(distinct tag_aai_lookup) as total',
													'from'		=> 'core_tags' ) );
			$total = $t['total'];
		}

		// Fetch records with tags
		$this->DB->build( array(	'select'	=> 'distinct tag_meta_id as meta_id, tag_meta_app as meta_app, tag_meta_area as meta_area',
									'from'		=> 'core_tags',
									'limit'		=> array( $start, $perCycle ) ) );
		$q = $this->DB->execute();

		$count = 0;
		while( $where = $this->DB->fetch($q) ) {
			$count++;

			// Build cache and insert
			$key		= md5( $where['meta_app'] . ';' . $where['meta_area'] . ';' . intval( $where['meta_id'] ) );
			$tagClass	= $where['meta_app'] . $where['meta_area'];

			// Get tag class for the current app/area.
			if ( ! $this->registry->isClassLoaded($tagClass) ) {
				require_once( IPS_ROOT_PATH . 'sources/classes/tags/bootstrap.php' );/*noLibHook*/
				$this->registry->setClass( $tagClass, classes_tags_bootstrap::run( $where['meta_app'], $where['meta_area'] ) );
			}

			// Load tags.
			$tags = $this->registry->getClass( $tagClass )->getRawTagsByMetaId( $where['meta_id'] );
			$text = array( 'tags' => array(), 'prefix' => '' );
			
			foreach( $tags as $id => $tag ) {
				$text['tags'][] = $tag['tag_text'];
				
				if ( $tag['tag_prefix'] ) {
					$text['prefix'] = $tag['tag_text'];
				}
			}

			// Don't create cache entry if tags and prefix empty
			if( !count( array_filter($text['tags']) ) && empty($text['prefix']) ) {
				continue;
			}
			
			$update = array( 'tag_cache_key'  => $key,
							 'tag_cache_text' => serialize( $text ),
							 'tag_cache_date' => IPS_UNIX_TIME_NOW
							);
			
			$this->DB->replace( 'core_tags_cache', $update, array( 'tag_cache_key' ) );

			$sum++;
		}

		// First iteration - log.
		if( $start == 0 ) {
			$this->registry->adminFunctions->saveAdminLog( $this->lang->words['pre_recache_log'] );
		}

		// Last iteration - all done.
		if( $count < $perCycle || $sum >= $total ) {
			$this->registry->output->redirect( $this->settings['base_url'] . $this->form_code, sprintf( $this->lang->words['pre_recache_msg'], $sum ) );
		}

		// If not done, show the update screen.
		$url  = "{$this->settings['base_url']}{$this->form_code}&do=rebuildCache&total=" . $total . "&st=" . ($start + $perCycle) . "&sum=" . $sum;
		$text = sprintf( $this->lang->words['pre_massadd_progress'], $start + $perCycle, sprintf( $this->lang->words['pre_recache_approx'], $total ), $url );
		$this->registry->output->html .= $this->registry->output->global_template->temporaryRedirect( $url, $text );
	}
}
